@@ -70,6 +70,7 @@ TQWidget* meepWidget = 0;
 TQWidget* meepWidgetP = 0;
 TQSlider* meepSlider = 0;
 TQTabBar* meepTabBar = 0;
+GdkGC* windowGC = 0;
 GdkGC* altBackGC = 0;
 TQWidget* smw = 0;
 
@@ -97,6 +98,8 @@ int isKeramik;
 int isAlloy;
 int isDomino;
 int isPolyester;
+int isMotif;
+int isCDE;
 int eclipseFix;
 int openOfficeFix;
 int mozillaFix;
@@ -346,7 +349,7 @@ void createTQApp()
 	int cmdlineAppNameLength = cmdlineAppName.length() + 1;
 
 	// Process hacks
-	mozillaFix = (cmdLine.contains("mozilla") || cmdLine.contains("firefox") || cmdLine.contains("iceweasel") || cmdLine.contains("thunderbird") || cmdLine.contains("icedove"));
+	mozillaFix = (cmdLine.contains("mozilla") || cmdLine.contains("firefox") || cmdLine.contains("iceweasel") || cmdLine.contains("thunderbird") || cmdLine.contains("icedove") || cmdLine.contains("palemoon"));
 	chromiumFix = (cmdLine.contains("chromium-browser") || cmdLine.contains("chrome-browser"));
 
 	openOfficeFix = (cmdLine.endsWith("soffice.bin"))
@@ -444,6 +447,8 @@ void createTQApp()
 	isAlloy     = (TQString(tqApp->style().name()).lower() == "alloy");
 	isDomino    = (TQString(tqApp->style().name()).lower() == "domino");
 	isPolyester = (TQString(tqApp->style().name()).lower() == "polyester");
+	isMotif     = (TQString(tqApp->style().name()).lower() == "motif");
+	isCDE       = (TQString(tqApp->style().name()).lower() == "cde");
 
 	if (isDomino)
 	{
@@ -505,6 +510,8 @@ void destroyTQApp()
 		delete tqApp;
 		tqApp = 0;
 	}
+	if (windowGC != 0)
+		gtk_gc_release(windowGC);
 	if (altBackGC != 0)
 		gtk_gc_release(altBackGC);
 
@@ -652,6 +659,36 @@ TQColor gdkColorToTQColor(GdkColor* c)
 	return TQColor(c->red / 256, c->green / 256, c->blue / 256);
 }
 
+void drawTQPixmapToWindow(GdkWindow* window, GdkGC* gc, TQPixmap* pixmap, int x, int y, int w, int h)
+{
+	GdkPixmap* pix;
+
+	// gdk_drawable_get_image crashes if requested region is outside of the window.
+	// Bitmap masks cause artifacts with Domino and Baghira.
+	int width, height;
+	gdk_drawable_get_size(window, &width, &height);
+	if (!pixmap->hasAlpha() || isDomino || isBaghira ||
+	    (width < x + w) || (height < y + h) || (0 > x) || (0 > y)) {
+		pix = gdk_pixmap_foreign_new(pixmap->handle());
+		gdk_draw_drawable(window, gc, pix, 0, 0, x, y, w, h);
+		g_object_unref(pix);
+		return;
+	}
+
+	// Gdk isn't aware of the TQPixmap mask,
+	// so instead we create a new pixmap from the window region and bitBlt over it.
+	if (windowGC == 0)
+		windowGC = gdk_gc_new(window);
+	TQPixmap gpixmap(w, h);
+	pix = gdk_pixmap_foreign_new(gpixmap.handle());
+	GdkImage* img = gdk_drawable_get_image(window, x, y, w, h);
+	gdk_draw_image(pix, windowGC, img, 0, 0, 0, 0, w, h);
+	bitBlt(&gpixmap, 0, 0, pixmap, 0, 0, w, h, TQt::CopyROP);
+
+	gdk_draw_drawable(window, gc, pix, 0, 0, x, y, w, h);
+	g_object_unref(img);
+	g_object_unref(pix);
+}
 
 // The drawing functions follow the same pattern:
 //  * Set the appropriate flags
@@ -762,9 +799,15 @@ void drawButton(GdkWindow* window, GtkStyle* style, GtkStateType state, int defa
 					tqApp->style().subRect(TQStyle::SR_PushButtonContents, &button),
 					button.colorGroup(), sflags);
 
-		GdkPixmap* pix = gdk_pixmap_foreign_new(pixmap.handle());
-		gdk_draw_drawable(window, style->bg_gc[state], pix, 0, 0, x, y, w, h);
-		g_object_unref(pix);
+		if (!isBaghira && !isMotif && !isCDE) {
+			TQBitmap bitmap(w, h, true);
+			TQPainter bpainter(&bitmap);
+			bpainter.setBrush(TQt::color1);
+			tqApp->style().drawControlMask(TQStyle::CE_PushButton, &bpainter, &button, TQRect(0,0,w,h), sflags);
+			pixmap.setMask(bitmap);
+		}
+
+		drawTQPixmapToWindow(window, style->bg_gc[state], &pixmap, x, y, w, h);
 	}
 	else {
 		TQPixmap     pixmap(w, h);
@@ -786,9 +829,15 @@ void drawButton(GdkWindow* window, GtkStyle* style, GtkStateType state, int defa
 		tqApp->style().drawControl(TQStyle::CE_PushButton, &painter, &button,
 		                          TQRect(0,0,w,h), button.palette().active(), sflags);
 
-		GdkPixmap* pix = gdk_pixmap_foreign_new(pixmap.handle());
-		gdk_draw_drawable(window, style->bg_gc[state], pix, 0, 0, x, y, w, h);
-		g_object_unref(pix);
+		if (!isBaghira && !isMotif && !isCDE) {
+			TQBitmap bitmap(w, h, true);
+			TQPainter bpainter(&bitmap);
+			bpainter.setBrush(TQt::color1);
+			tqApp->style().drawControlMask(TQStyle::CE_PushButton, &bpainter, &button, TQRect(0,0,w,h), sflags);
+			pixmap.setMask(bitmap);
+		}
+
+		drawTQPixmapToWindow(window, style->bg_gc[state], &pixmap, x, y, w, h);
 	}
 }
 
@@ -1066,6 +1115,11 @@ void drawCheckBox(GdkWindow* window, GtkStyle* style, GtkStateType state, int ch
 	int realH = tqApp->style().pixelMetric(TQStyle::PM_IndicatorHeight);
 	int realW = tqApp->style().pixelMetric(TQStyle::PM_IndicatorWidth);
 
+	if (isKeramik) {
+		realW-= 2;
+		realH-= 2;
+	}
+
 	if ((realW < 1) || (realH < 1))
 		return;
 
@@ -1084,9 +1138,13 @@ void drawCheckBox(GdkWindow* window, GtkStyle* style, GtkStateType state, int ch
 	int xOffset = (realW - w) / 2;
 	int yOffset = (realH - h) / 2;
 
-	GdkPixmap* pix = gdk_pixmap_foreign_new(pixmap.handle());
-	gdk_draw_drawable(window, style->bg_gc[state], pix, 0, 0, x - xOffset, y - yOffset, realW, realH);
-	g_object_unref(pix);
+	TQBitmap bitmap(realW, realH, true);
+	TQPainter bpainter(&bitmap);
+	bpainter.setBrush(TQt::color1);
+	tqApp->style().drawControlMask(TQStyle::CE_CheckBox, &bpainter, &checkbox, TQRect(0,0,realW,realH), sflags);
+	pixmap.setMask(bitmap);
+
+	drawTQPixmapToWindow(window, style->bg_gc[state], &pixmap, x, y, w, h);
 }
 
 void drawMenuCheck(GdkWindow* window, GtkStyle* style, GtkStateType state, int x, int y, int w, int h)
@@ -1135,7 +1193,7 @@ void drawRadioButton(GdkWindow* window, GtkStyle* style, GtkStateType state, int
 	if ((realW < 1) || (realH < 1))
 		return;
 
-	TQPixmap pixmap(realH, realW);
+	TQPixmap pixmap(realW, realH);
 	TQPainter painter(&pixmap);
 	TQRadioButton radio(0);
 
@@ -1149,16 +1207,20 @@ void drawRadioButton(GdkWindow* window, GtkStyle* style, GtkStateType state, int
         else
                 painter.fillRect(0, 0, realW, realH, tqApp->palette().active().brush(TQColorGroup::Background));
 
-	tqApp->style().drawControl(TQStyle::CE_RadioButton, &painter, &radio, TQRect(0,0,realH,realW), tqApp->palette().active(), sflags);
+	tqApp->style().drawControl(TQStyle::CE_RadioButton, &painter, &radio, TQRect(0,0,realW,realH), tqApp->palette().active(), sflags);
 
 	// TQt checkboxes are usually bigger than GTK wants.
 	// We cheat, and draw them over the expected area.
 	int xOffset = (realW - w) / 2;
 	int yOffset = (realH - h) / 2;
 
-	GdkPixmap* pix = gdk_pixmap_foreign_new(pixmap.handle());
-	gdk_draw_drawable(window, style->bg_gc[state], pix, 0, 0, x - xOffset, y - yOffset, realW, realH);
-	g_object_unref(pix);
+	TQBitmap bitmap(realW, realH, true);
+	TQPainter bpainter(&bitmap);
+	bpainter.setBrush(TQt::color1);
+	tqApp->style().drawControlMask(TQStyle::CE_RadioButton, &bpainter, &radio, TQRect(0,0,realW,realH), sflags);
+	pixmap.setMask(bitmap);
+
+	drawTQPixmapToWindow(window, style->bg_gc[state], &pixmap, x-xOffset, y-yOffset, realW, realH);
 }
 
 
@@ -1172,12 +1234,35 @@ void drawScrollBarSlider(GdkWindow* window, GtkStyle* style, GtkStateType state,
 
 	int wCorrected = w;
 	int hCorrected = h;
+	int xOffset = 0;
+	int yOffset = 0;
 	if (isDomino)
 	{
 		if (orientation == GTK_ORIENTATION_HORIZONTAL)
 		    wCorrected = w + 14;
 		else
 		    hCorrected = h + 14;
+	} else if (isKeramik) {
+		if (orientation == GTK_ORIENTATION_HORIZONTAL) {
+			wCorrected = w + 3;
+			hCorrected = h - 1;
+			yOffset = 1;
+		} else {
+			wCorrected = w - 1;
+			hCorrected = h + 3;
+			xOffset = 1;
+		}
+	}
+	if (mozillaFix) {
+		if (orientation == GTK_ORIENTATION_HORIZONTAL) {
+			hCorrected-= 2;
+			wCorrected-= 1;
+			yOffset+= 1;
+		} else {
+			wCorrected-= 2;
+			hCorrected-= 1;
+			xOffset+= 1;
+		}
 	}
 	TQPixmap pixmap(wCorrected, hCorrected);
 	TQPainter painter(&pixmap);
@@ -1239,7 +1324,7 @@ void drawScrollBarSlider(GdkWindow* window, GtkStyle* style, GtkStateType state,
 			gdk_draw_drawable(window, style->bg_gc[state], pix, 0, endsSkip, x, y, w, h-1);
 	}
 	else
-		gdk_draw_drawable(window, style->bg_gc[state], pix, 0, 0, x, y, w, h);
+		gdk_draw_drawable(window, style->bg_gc[state], pix, 0, 0, x+xOffset, y+yOffset, w, h);
 	g_object_unref(pix);
 }
 
@@ -1604,6 +1689,7 @@ void drawProgressBar(GdkWindow * window, GtkStyle * style, GtkStateType state, G
 	bar.setPercentageVisible(false);
 
 	TQPixmap pixmap = TQPixmap::grabWidget(&bar);
+	pixmap.setMask(pixmap.createHeuristicMask());
 
 	TQWMatrix matrix;
 	switch (orientation)
@@ -1617,9 +1703,7 @@ void drawProgressBar(GdkWindow * window, GtkStyle * style, GtkStateType state, G
 	if (orientation != GTK_PROGRESS_LEFT_TO_RIGHT)
 		pixmap = pixmap.xForm(matrix);
 
-	GdkPixmap* pix = gdk_pixmap_foreign_new(pixmap.handle());
-	gdk_draw_drawable(window, style->bg_gc[state], pix, 0, 0, x, y, w, h);
-	g_object_unref(pix);
+	drawTQPixmapToWindow(window, style->bg_gc[state], &pixmap, x, y, w, h);
 }
 
 void drawSlider(GdkWindow * window, GtkStyle * style, GtkStateType state, GtkAdjustment *adj, int x, int y, int w, int h, GtkOrientation orientation, int inverted)
@@ -1642,9 +1726,8 @@ void drawSlider(GdkWindow * window, GtkStyle * style, GtkStateType state, GtkAdj
 		meepSlider->setValue(100-(int)((adj->value-adj->lower)/(adj->upper-adj->lower)*100));
 
 	TQPixmap pixmap = TQPixmap::grabWidget(meepSlider);
-	GdkPixmap* pix = gdk_pixmap_foreign_new(pixmap.handle());
-	gdk_draw_drawable(window, style->bg_gc[state], pix, 0, 0, x, y, w, h);
-	g_object_unref(pix);
+	pixmap.setMask(pixmap.createHeuristicMask());
+	drawTQPixmapToWindow(window, style->bg_gc[state], &pixmap, x, y, w, h);
 }
 
 void drawSpinButton(GdkWindow * window, GtkStyle * style, GtkStateType state, int direction, int x, int y, int w, int h)
@@ -1663,9 +1746,8 @@ void drawSpinButton(GdkWindow * window, GtkStyle * style, GtkStateType state, in
 	painter.fillRect(0, 0, w, h, tqApp->palette().active().brush(TQColorGroup::Background));
 	tqApp->style().drawPrimitive((direction ? TQStyle::PE_SpinWidgetDown : TQStyle::PE_SpinWidgetUp), &painter, TQRect(0,0,w,h), tqApp->palette().active(), sflags);
 
-	GdkPixmap* pix = gdk_pixmap_foreign_new(pixmap.handle());
-	gdk_draw_drawable(window, style->bg_gc[state], pix, 0, 0, x, y, w, h);
-	g_object_unref(pix);
+	pixmap.setMask(pixmap.createHeuristicMask());
+	drawTQPixmapToWindow(window, style->bg_gc[state], &pixmap, x, y, w, h);
 }
 
 void drawListHeader(GdkWindow* window, GtkStyle* style, GtkStateType state, int x, int y, int w, int h)
@@ -1860,9 +1942,8 @@ void drawArrow(GdkWindow* window, GtkStyle* style, GtkStateType state, GtkArrowT
 
 	tqApp->style().drawPrimitive(element, &painter, TQRect(0,0,w,h), tqApp->palette().active(), sflags);
 
-	GdkPixmap* pix = gdk_pixmap_foreign_new(pixmap.handle());
-	gdk_draw_drawable(window, style->bg_gc[state], pix, 0, 0, x, y, w, h);
-	g_object_unref(pix);
+	pixmap.setMask(pixmap.createHeuristicMask());
+	drawTQPixmapToWindow(window, style->bg_gc[state], &pixmap, x, y, w, h);
 }
 
 void drawFocusRect(GdkWindow * window, GtkStyle * style, int x, int y, int w, int h)
